@@ -6,39 +6,53 @@ Copyright (c) 1999-2002 Ng Pheng Siong. All rights reserved."""
 
 import logging
 import os
+import platform
 import tempfile
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import ctypes
+if platform.system() == 'Windows':
+    import ctypes.wintypes
 
 from M2Crypto.BIO import File, openfile
+from . import unittest
 
 log = logging.getLogger(__name__)
 
 
+def getCountProcHandles():
+    PROCESS_QUERY_INFORMATION = 0x400
+    handle = ctypes.windll.kernel32.OpenProcess(
+        PROCESS_QUERY_INFORMATION, 0, os.getpid())
+    hndcnt = ctypes.wintypes.DWORD()
+    ctypes.windll.kernel32.GetProcessHandleCount(
+        handle, ctypes.byref(hndcnt))
+    sys_value = hndcnt.value
+    ctypes.windll.kernel32.CloseHandle(handle)
+    return sys_value + 1
+
+
 class FileTestCase(unittest.TestCase):
 
     def setUp(self):
         self.data = b'abcdef' * 64
         self.fd, self.fname = tempfile.mkstemp()
 
-        # mvyskocil: to check leaks of file descriptors
-        self._proc = "/proc/{0}/fd/".format(os.getpid())
-        self.max_fd = self.mfd()
+        if platform.system() in ['Linux', 'Darwin', 'FreeBSD']:
+            self.__dev_fd = "/dev/fd/"
 
-    # FIXME: this indeed does work on Linux and probably other *nixes,
-    # but definitelly
-    #       not on windows, add a fallback method, like
-    #       os.fdopen().fileno()-1
-    def mfd(self):
-        return int(os.listdir(self._proc)[-1])
+        self.fd_count = self.__mfd()
+
+    def __mfd(self):
+        if hasattr(self, '__dev_fd'):
+            return len(os.listdir(self.__dev_fd))
+        elif platform.system() == 'Windows':
+            return getCountProcHandles()
+        else:
+            return None
 
     def tearDown(self):
 
-        self.assertEqual(
-            self.max_fd, self.mfd(),
-            "last test did not close all file descriptors properly")
+        self.assertEqual(self.fd_count, self.__mfd(),
+                         "last test did not close all file descriptors properly")
 
         try:
             os.close(self.fd)
@@ -88,15 +102,30 @@ class FileTestCase(unittest.TestCase):
         self.assertEqual(len(in_data), len(self.data))
         self.assertEqual(in_data, self.data)
 
+    def test_readline_bin(self):
+        with open(self.fname, 'wb') as f:
+            f.write(b'hello\nworld\n')
+        with openfile(self.fname, 'rb') as f:
+            self.assertTrue(f.readable())
+            self.assertEqual(f.readline(), b'hello\n')
+            self.assertEqual(f.readline(), b'world\n')
+        with openfile(self.fname, 'rb') as f:
+            self.assertEqual(
+                f.readlines(),
+                [b'hello\n', b'world\n'])
+
     def test_readline(self):
+        sep = os.linesep.encode()
         with open(self.fname, 'w') as f:
             f.write('hello\nworld\n')
-        with openfile(self.fname, 'r') as f:
+        with openfile(self.fname, 'rb') as f:
             self.assertTrue(f.readable())
-            self.assertEqual(f.readline(), 'hello\n')
-            self.assertEqual(f.readline(), 'world\n')
-        with openfile(self.fname, 'r') as f:
-            self.assertEqual(f.readlines(), ['hello\n', 'world\n'])
+            self.assertEqual(f.readline(), b'hello' + sep)
+            self.assertEqual(f.readline(), b'world' + sep)
+        with openfile(self.fname, 'rb') as f:
+            self.assertEqual(
+                f.readlines(),
+                [b'hello' + sep, b'world' + sep])
 
     def test_tell_seek(self):
         with open(self.fname, 'w') as f:
@@ -108,7 +137,7 @@ class FileTestCase(unittest.TestCase):
 
 
 def suite():
-    return unittest.makeSuite(FileTestCase)
+    return unittest.TestLoader().loadTestsFromTestCase(FileTestCase)
 
 
 if __name__ == '__main__':
