@@ -4,12 +4,8 @@
 
 Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""
 
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-
 from M2Crypto import DH, BIO, Rand
+from . import unittest
 
 
 class DHTestCase(unittest.TestCase):
@@ -63,7 +59,7 @@ class DHTestCase(unittest.TestCase):
 
 
 def suite():
-    return unittest.makeSuite(DHTestCase)
+    return unittest.TestLoader().loadTestsFromTestCase(DHTestCase)
 
 
 if __name__ == '__main__':
