@@ -21,12 +21,8 @@ from __future__ import absolute_import
 """
 import logging
 
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-
 from M2Crypto import EC, Rand, m2  # noqa
+from . import unittest
 
 log = logging.getLogger(__name__)
 
@@ -165,7 +161,7 @@ class ECCurveTests(unittest.TestCase):
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(ECCurveTests))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ECCurveTests))
     return suite
 
 
