@@ -6,13 +6,10 @@ Copyright (c) 2000 Ng Pheng Siong. All rights reserved.
 Portions copyright (c) 2005-2006 Vrije Universiteit Amsterdam. All
 rights reserved.
 """
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
 from M2Crypto import EC, Rand
 
+from . import unittest
 from tests.test_ec_curves import tested_curve
 
 
@@ -45,7 +42,7 @@ class ECDHTestCase(unittest.TestCase):
 
 
 def suite():
-    return unittest.makeSuite(ECDHTestCase)
+    return unittest.TestLoader().loadTestsFromTestCase(ECDHTestCase)
 
 
 if __name__ == '__main__':
