@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from __future__ import absolute_import, division
 
 """
@@ -8,30 +7,44 @@ Copyright (c) 2004-2007 Open Source Applications Foundation
 Author: Heikki Toivonen
 """
 
-import codecs
+import base64
 import hashlib
 import io
 import logging
 
-from M2Crypto import BIO, EVP, RSA, Rand, m2, six, util
-
 from binascii import a2b_hex, hexlify, unhexlify
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
+from parameterized import parameterized
+
+from M2Crypto import BIO, EVP, RSA, Rand, m2, util
+from . import unittest
 from tests.fips import fips_mode
 
 log = logging.getLogger('test_EVP')
 
+ciphers = [
+    'des_ede_ecb', 'des_ede_cbc', 'des_ede_cfb', 'des_ede_ofb',
+    'des_ede3_ecb', 'des_ede3_cbc', 'des_ede3_cfb', 'des_ede3_ofb',
+    'aes_128_ecb', 'aes_128_cbc', 'aes_128_cfb', 'aes_128_ofb',
+    'aes_128_ctr', 'aes_192_ecb', 'aes_192_cbc', 'aes_192_cfb',
+    'aes_192_ofb', 'aes_192_ctr', 'aes_256_ecb', 'aes_256_cbc',
+    'aes_256_cfb', 'aes_256_ofb', 'aes_256_ctr']
+nonfips_ciphers = ['bf_ecb', 'bf_cbc', 'bf_cfb', 'bf_ofb',
+                   # 'idea_ecb', 'idea_cbc', 'idea_cfb', 'idea_ofb',
+                   'cast5_ecb', 'cast5_cbc', 'cast5_cfb', 'cast5_ofb',
+                   # 'rc5_ecb', 'rc5_cbc', 'rc5_cfb', 'rc5_ofb',
+                   'des_ecb', 'des_cbc', 'des_cfb', 'des_ofb',
+                   'rc4', 'rc2_40_cbc']
+if not fips_mode:  # Disabled algorithms
+    ciphers += nonfips_ciphers
+
 
 class EVPTestCase(unittest.TestCase):
     def _gen_callback(self, *args):
         pass
 
     def _pass_callback(self, *args):
-        return 'foobar'
+        return b'foobar'
 
     def _assign_rsa(self):
         rsa = RSA.gen_key(1024, 3, callback=self._gen_callback)
@@ -43,14 +56,15 @@ class EVPTestCase(unittest.TestCase):
         rsa = self._assign_rsa()
         rsa.check_key()
 
-    @unittest.skipIf(six.PY3,
-                     "test_pem hangs under python3 and is not yet fixed")
     def test_pem(self):
         rsa = RSA.gen_key(1024, 3, callback=self._gen_callback)
         pkey = EVP.PKey()
         pkey.assign_rsa(rsa)
-        self.assertNotEqual(pkey.as_pem(callback=self._pass_callback),
-                            pkey.as_pem(cipher=None))
+
+        result_w_callback = pkey.as_pem(callback=self._pass_callback)
+        result_wo_callback = pkey.as_pem(cipher=None)
+        self.assertNotEqual(result_w_callback, result_wo_callback)
+
         with self.assertRaises(ValueError):
             pkey.as_pem(cipher='noXX$$%%suchcipher',
                         callback=self._pass_callback)
@@ -68,7 +82,8 @@ class EVPTestCase(unittest.TestCase):
         self.assertEqual(len(der_blob), 160)
 
     def test_get_digestbyname(self):
-        self.assertEqual(m2.get_digestbyname('sha513'), None)
+        with self.assertRaises(EVP.EVPError):
+            m2.get_digestbyname('sha513')
         self.assertNotEqual(m2.get_digestbyname('sha1'), None)
 
     def test_MessageDigest(self):  # noqa
@@ -153,8 +168,6 @@ class EVPTestCase(unittest.TestCase):
         with self.assertRaises(ValueError):
             EVP.hmac(b'key', b'data', algo='sha513')
 
-    @unittest.skipIf(six.PY3,
-                     "test_get_rsa hangs under python3 and is not yet fixed")
     def test_get_rsa(self):
         """
         Testing retrieving the RSA key from the PKey instance.
@@ -185,8 +198,6 @@ class EVPTestCase(unittest.TestCase):
         rsa3 = RSA.gen_key(1024, 3, callback=self._gen_callback)
         self.assertNotEqual(rsa.sign(digest), rsa3.sign(digest))
 
-    @unittest.skipIf(six.PY3,
-                     'FIXME test_load_key_string_pubkey hangs under python3')
     def test_load_key_string_pubkey(self):
         """
         Testing creating a PKey instance from PEM string.
@@ -255,6 +266,62 @@ class EVPTestCase(unittest.TestCase):
         pubkey.verify_update(b'test  message not')
         self.assertEqual(pubkey.verify_final(sig), 0)
 
+    @unittest.skipIf(m2.OPENSSL_VERSION_NUMBER < 0x10101000,
+                     'Relies on support for Ed25519 which was introduced in OpenSSL 1.1.1')
+    def test_digest_verify(self):
+        pkey = EVP.load_key('tests/ed25519.priv.pem')
+        pkey.reset_context(None)
+        pkey.digest_sign_init()
+        sig = pkey.digest_sign(b'test  message')
+
+        # OK
+        pkey = EVP.load_key_pubkey('tests/ed25519.pub.pem')
+        pkey.reset_context(None)
+        pkey.digest_verify_init()
+        self.assertEqual(pkey.digest_verify(sig, b'test  message'), 1)
+
+        # wrong public key
+        pkey = EVP.load_key_pubkey('tests/ed25519.pub2.pem')
+        pkey.reset_context(None)
+        pkey.digest_verify_init()
+        self.assertEqual(pkey.digest_verify(sig, b'test  message'), 0)
+
+        # wrong message
+        pkey = EVP.load_key_pubkey('tests/ed25519.pub.pem')
+        pkey.reset_context(None)
+        pkey.digest_verify_init()
+        self.assertEqual(pkey.digest_verify(sig, b'test  message  not'), 0)
+
+    @unittest.skipIf(m2.OPENSSL_VERSION_NUMBER < 0x90800F or m2.OPENSSL_NO_EC != 0,
+                     'Relies on support for EC')
+    def test_digest_verify_final(self):
+        pkey = EVP.load_key('tests/ec.priv.pem')
+        pkey.reset_context('sha256')
+        pkey.digest_sign_init()
+        pkey.digest_sign_update(b'test  message')
+        sig = pkey.digest_sign_final()
+
+        # OK
+        pkey = EVP.load_key_pubkey('tests/ec.pub.pem')
+        pkey.reset_context('sha256')
+        pkey.digest_verify_init()
+        pkey.digest_verify_update(b'test  message')
+        self.assertEqual(pkey.digest_verify_final(sig), 1)
+
+        # wrong public key
+        pkey = EVP.load_key_pubkey('tests/ec.pub2.pem')
+        pkey.reset_context('sha256')
+        pkey.digest_verify_init()
+        pkey.digest_verify_update(b'test  message')
+        self.assertEqual(pkey.digest_verify_final(sig), 0)
+
+        # wrong message
+        pkey = EVP.load_key_pubkey('tests/ec.pub.pem')
+        pkey.reset_context('sha256')
+        pkey.digest_verify_init()
+        pkey.digest_verify_update(b'test  message not')
+        self.assertEqual(pkey.digest_verify_final(sig), 0)
+
     def test_load_bad(self):
         with self.assertRaises(BIO.BIOError):
             EVP.load_key('thisdoesnotexist-dfgh56789')
@@ -279,10 +346,10 @@ class EVPTestCase(unittest.TestCase):
             SIGN_PRIVATE.sign_init()
             SIGN_PRIVATE.sign_update(data)
             signed_data = SIGN_PRIVATE.sign_final()
-            return codecs.encode(signed_data, 'base64')
+            return base64.b64encode(signed_data)
 
         def verify(response):
-            signature = codecs.decode(response['sign'], 'base64')
+            signature = base64.b64decode(response['sign'])
             data = response['data']
             verify_evp = EVP.PKey()
             # capture parameter on the following line is required by
@@ -300,6 +367,7 @@ class EVPTestCase(unittest.TestCase):
         self.assertTrue(verify(res))  # works fine
         self.assertTrue(verify(res))  # segmentation fault in *verify_final*
 
+
 class CipherTestCase(unittest.TestCase):
     def cipher_filter(self, cipher, inf, outf):
         while 1:
@@ -333,50 +401,35 @@ class CipherTestCase(unittest.TestCase):
 
         self.assertEqual(otxt, ptxt, '%s algorithm cipher test failed' % algo)
 
-    def test_ciphers(self):
-        ciphers = [
-            'des_ede_ecb', 'des_ede_cbc', 'des_ede_cfb', 'des_ede_ofb',
-            'des_ede3_ecb', 'des_ede3_cbc', 'des_ede3_cfb', 'des_ede3_ofb',
-            'aes_128_ecb', 'aes_128_cbc', 'aes_128_cfb', 'aes_128_ofb',
-            'aes_128_ctr', 'aes_192_ecb', 'aes_192_cbc', 'aes_192_cfb',
-            'aes_192_ofb', 'aes_192_ctr', 'aes_256_ecb', 'aes_256_cbc',
-            'aes_256_cfb', 'aes_256_ofb', 'aes_256_ctr']
-        nonfips_ciphers = ['bf_ecb', 'bf_cbc', 'bf_cfb', 'bf_ofb',
-                           # 'idea_ecb', 'idea_cbc', 'idea_cfb', 'idea_ofb',
-                           'cast5_ecb', 'cast5_cbc', 'cast5_cfb', 'cast5_ofb',
-                           # 'rc5_ecb', 'rc5_cbc', 'rc5_cfb', 'rc5_ofb',
-                           'des_ecb', 'des_cbc', 'des_cfb', 'des_ofb',
-                           'rc4', 'rc2_40_cbc']
-        if not fips_mode:  # Disabled algorithms
-            ciphers += nonfips_ciphers
-        for i in ciphers:
-            self.try_algo(i)
+    @parameterized.expand(ciphers)
+    def test_ciphers(self, ciph):
+        self.try_algo(ciph)
 
-        # idea might not be compiled in
-        ciphers = ['idea_ecb', 'idea_cbc', 'idea_cfb', 'idea_ofb']
-        try:
-            for i in ciphers:
-                self.try_algo(i)
-        except ValueError as e:
-            if str(e) != "('unknown cipher', 'idea_ecb')":
-                raise
+    # # non-compiled (['idea_ecb', 'idea_cbc', 'idea_cfb', 'idea_ofb'])
+    # @parameterized.expand([], skip_on_empty=True)
+    # def test_ciphers_not_compiled_idea(self, ciph):
+    #     # idea might not be compiled in
+    #     try:
+    #         self.try_algo(ciph)
+    #     except ValueError as e:
+    #         if str(e) != "('unknown cipher', 'idea_ecb')":
+    #             raise
 
-        # rc5 might not be compiled in
-        ciphers = ['rc5_ecb', 'rc5_cbc', 'rc5_cfb', 'rc5_ofb']
-        try:
-            for i in ciphers:
-                self.try_algo(i)
-        except ValueError as e:
-            if str(e) != "('unknown cipher', 'rc5_ecb')":
-                raise
+    # # ['rc5_ecb', 'rc5_cbc', 'rc5_cfb', 'rc5_ofb']
+    # @parameterized.expand([], skip_on_empty=True)
+    # def test_ciphers_not_compiled_rc5(self, ciph):
+    #     # rc5 might not be compiled in
+    #     try:
+    #         self.try_algo(ciph)
+    #     except ValueError as e:
+    #         if str(e) != "('unknown cipher', 'rc5_ofb')":
+    #             raise
 
+    def test_ciphers_nosuch(self):
         with self.assertRaises(ValueError):
             self.try_algo('nosuchalgo4567')
 
-    def test_AES(self):  # noqa
-        enc = 1
-        dec = 0
-        tests = [
+    @parameterized.expand([
             # test vectors from rfc 3602
             # Case #1: Encrypting 16 bytes (1 block) using AES-CBC with
             # 128-bit key
@@ -404,54 +457,55 @@ class CipherTestCase(unittest.TestCase):
                 'PT': b'This is a 48-byte message (exactly 3 AES blocks)',
                 'CT': b'd0a02b3836451753d493665d33f0e8862dea54cdb293abc7506939276772f8d5021c19216bad525c8579695d83ba2684',
             },
-        ]
+            ], name_func=lambda func, num, par: "{}_{}".format(func, num))
+    def test_AES(self, test):  # noqa
+        enc = 1
+        dec = 0
 
         # Test with padding
-        for test in tests:
-            # encrypt
-            k = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
-                           iv=unhexlify(test['IV']), op=enc)
-            pbuf = io.BytesIO(test['PT'])
-            cbuf = io.BytesIO()
-            ciphertext = hexlify(self.cipher_filter(k, pbuf, cbuf))
-            cipherpadding = ciphertext[len(test['PT']) * 2:]
-            # Remove the padding from the end
-            ciphertext = ciphertext[:len(test['PT']) * 2]
-            pbuf.close()
-            cbuf.close()
-            self.assertEqual(ciphertext, test['CT'])
+        # encrypt
+        k = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
+                       iv=unhexlify(test['IV']), op=enc)
+        pbuf = io.BytesIO(test['PT'])
+        cbuf = io.BytesIO()
+        ciphertext = hexlify(self.cipher_filter(k, pbuf, cbuf))
+        cipherpadding = ciphertext[len(test['PT']) * 2:]
+        # Remove the padding from the end
+        ciphertext = ciphertext[:len(test['PT']) * 2]
+        pbuf.close()
+        cbuf.close()
+        self.assertEqual(ciphertext, test['CT'])
 
-            # decrypt
-            j = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
-                           iv=unhexlify(test['IV']), op=dec)
-            pbuf = io.BytesIO()
-            cbuf = io.BytesIO(unhexlify(test['CT'] + cipherpadding))
-            plaintext = self.cipher_filter(j, cbuf, pbuf)
-            pbuf.close()
-            cbuf.close()
-            self.assertEqual(plaintext, test['PT'])
+        # decrypt
+        j = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
+                       iv=unhexlify(test['IV']), op=dec)
+        pbuf = io.BytesIO()
+        cbuf = io.BytesIO(unhexlify(test['CT'] + cipherpadding))
+        plaintext = self.cipher_filter(j, cbuf, pbuf)
+        pbuf.close()
+        cbuf.close()
+        self.assertEqual(plaintext, test['PT'])
 
         # Test without padding
-        for test in tests:
-            # encrypt
-            k = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
-                           iv=unhexlify(test['IV']), op=enc, padding=False)
-            pbuf = io.BytesIO(test['PT'])
-            cbuf = io.BytesIO()
-            ciphertext = hexlify(self.cipher_filter(k, pbuf, cbuf))
-            pbuf.close()
-            cbuf.close()
-            self.assertEqual(ciphertext, test['CT'])
+        # encrypt
+        k = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
+                       iv=unhexlify(test['IV']), op=enc, padding=False)
+        pbuf = io.BytesIO(test['PT'])
+        cbuf = io.BytesIO()
+        ciphertext = hexlify(self.cipher_filter(k, pbuf, cbuf))
+        pbuf.close()
+        cbuf.close()
+        self.assertEqual(ciphertext, test['CT'])
 
-            # decrypt
-            j = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
-                           iv=unhexlify(test['IV']), op=dec, padding=False)
-            pbuf = io.BytesIO()
-            cbuf = io.BytesIO(unhexlify(test['CT']))
-            plaintext = self.cipher_filter(j, cbuf, pbuf)
-            pbuf.close()
-            cbuf.close()
-            self.assertEqual(plaintext, test['PT'])
+        # decrypt
+        j = EVP.Cipher(alg='aes_128_cbc', key=unhexlify(test['KEY']),
+                       iv=unhexlify(test['IV']), op=dec, padding=False)
+        pbuf = io.BytesIO()
+        cbuf = io.BytesIO(unhexlify(test['CT']))
+        plaintext = self.cipher_filter(j, cbuf, pbuf)
+        pbuf.close()
+        cbuf.close()
+        self.assertEqual(plaintext, test['PT'])
 
     def test_AES_ctr(self):  # noqa
         # In CTR mode, encrypt and decrypt are actually the same
@@ -472,7 +526,6 @@ class CipherTestCase(unittest.TestCase):
 
         for key_size in [128, 192, 256]:
             alg = 'aes_%s_ctr' % str(key_size)
-            log.info('Testing cipher %s', alg)
 
             # Our key for this test is 256 bits in length (32 bytes).
             # We will trim it to the appopriate length for testing AES-128
@@ -525,6 +578,11 @@ class CipherTestCase(unittest.TestCase):
                 unhexlify('5cd148eeaf680d4ff933aed83009cad4110162f53ef89fd44fad09611b0524d4'),
                 unhexlify(''))
 
+    def test_cipher_init_reinit(self):
+        ctx = m2.cipher_ctx_new()
+        m2.cipher_init(ctx, m2.aes_128_cbc(), b'\x01' * (128//8), b'\x02' * (128//8), 1)
+        m2.cipher_init(ctx, m2.aes_128_cbc(), None, None, 1)
+
 
 class PBKDF2TestCase(unittest.TestCase):
     def test_rfc3211_test_vectors(self):
@@ -631,12 +689,13 @@ class HMACTestCase(unittest.TestCase):
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(EVPTestCase))
-    suite.addTest(unittest.makeSuite(CipherTestCase))
-    suite.addTest(unittest.makeSuite(PBKDF2TestCase))
-    suite.addTest(unittest.makeSuite(HMACTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(EVPTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(CipherTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(PBKDF2TestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(HMACTestCase))
     return suite
 
+
 if __name__ == '__main__':
     Rand.load_file('randpool.dat', -1)
     unittest.TextTestRunner().run(suite())
