@@ -8,13 +8,9 @@ Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""
 import hashlib
 import logging
 import os
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
 from M2Crypto import BIO, RSA, Rand, X509, m2, six
-
+from . import unittest
 from tests.fips import fips_mode
 
 log = logging.getLogger('test_RSA')
@@ -42,18 +38,20 @@ class RSATestCase(unittest.TestCase):
 
     def pp_callback(self, *args):
         # The passphrase for rsa.priv2.pem is 'qwerty'.
-        return 'qwerty'
+        return b'qwerty'
 
     def pp2_callback(self, *args):
         # Misbehaving passphrase callback.
-        pass
+        return b'blabla'
+
+    def test_rsa_exceptions(self):
+        with self.assertRaises(RSA.RSAError):
+            RSA.rsa_error()
 
     def test_loadkey_junk(self):
         with self.assertRaises(RSA.RSAError):
             RSA.load_key(self.errkey)
 
-    @unittest.skipIf(six.PY3,
-                     'test_loadkey_pp hangs under python3')
     def test_loadkey_pp(self):
         rsa = RSA.load_key(self.privkey2, self.pp_callback)
         self.assertEqual(len(rsa), 1024)
@@ -61,7 +59,6 @@ class RSATestCase(unittest.TestCase):
                          b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
-    @unittest.skipIf(six.PY3, 'test_loadkey_pp_bad_cp hangs under python3')
     def test_loadkey_pp_bad_cb(self):
         with self.assertRaises(RSA.RSAError):
             RSA.load_key(self.privkey2, self.pp2_callback)
@@ -116,6 +113,8 @@ class RSATestCase(unittest.TestCase):
         with self.assertRaises(TypeError):
             priv.private_encrypt(self.gen_callback, RSA.pkcs1_padding)
 
+    @unittest.skipIf(m2.OPENSSL_VERSION_NUMBER < 0x1010103f,
+                     'Relies on fix which happened only in OpenSSL 1.1.1c')
     def test_public_encrypt(self):
         priv = RSA.load_key(self.privkey)
         # pkcs1_padding, pkcs1_oaep_padding
@@ -127,13 +126,11 @@ class RSATestCase(unittest.TestCase):
 
         # sslv23_padding
         ctxt = priv.public_encrypt(self.data, RSA.sslv23_padding)
-        with self.assertRaises(RSA.RSAError):
-            priv.private_decrypt(ctxt, RSA.sslv23_padding)
-        with self.assertRaises(RSA.RSAError):
-            priv.private_decrypt(ctxt, RSA.sslv23_padding)
+        res = priv.private_decrypt(ctxt, RSA.sslv23_padding)
+        self.assertEqual(res, self.data)
 
         # no_padding
-        with self.assertRaises(RSA.RSAError):
+        with six.assertRaisesRegex(self, RSA.RSAError, 'data too small'):
             priv.public_encrypt(self.data, RSA.no_padding)
 
         # Type-check the data to be encrypted.
@@ -348,7 +345,7 @@ class RSATestCase(unittest.TestCase):
 
 
 def suite():
-    return unittest.makeSuite(RSATestCase)
+    return unittest.TestLoader().loadTestsFromTestCase(RSATestCase)
 
 
 if __name__ == '__main__':
