@@ -13,13 +13,10 @@ import base64
 import logging
 import os
 import time
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 import warnings
 
-from M2Crypto import ASN1, BIO, EVP, RSA, Rand, X509, m2  # noqa
+from M2Crypto import ASN1, BIO, EVP, RSA, Rand, X509, m2, six  # noqa
+from . import unittest
 
 log = logging.getLogger(__name__)
 
@@ -30,7 +27,7 @@ class X509TestCase(unittest.TestCase):
         pass
 
     def setUp(self):
-        self.expected_hash = 'F0429E180321BB3C0CBEAB811360BB0E27CC0259'
+        self.expected_hash = 'BA4212E8B55527570828E7F5A0005D17C64BDC4C'
 
     def mkreq(self, bits, ca=0):
         pk = EVP.PKey()
@@ -198,7 +195,7 @@ class X509TestCase(unittest.TestCase):
         #        /compound_stmts.html#the-for-statement
         # so this checks what are the attributes of the last value of
         # ``cn`` variable.
-        cn.set_data("Hello There!")
+        cn.set_data(b"Hello There!")
         self.assertEqual(cn.get_data().as_text(), "Hello There!")
 
         # OpenSSL 1.0.1h switched from encoding strings as PRINTABLESTRING (the
@@ -541,22 +538,25 @@ class X509TestCase(unittest.TestCase):
         os.remove('tests/tmpcert.der')
 
     def test_malformed_data(self):
-        with self.assertRaises(X509.X509Error):
-            X509.load_cert_string('Hello')
-        with self.assertRaises(X509.X509Error):
-            X509.load_cert_der_string('Hello')
-        with self.assertRaises(X509.X509Error):
-            X509.new_stack_from_der('Hello')
-        with self.assertRaises(X509.X509Error):
-            X509.load_cert('tests/alltests.py')
-        with self.assertRaises(X509.X509Error):
-            X509.load_request('tests/alltests.py')
-        with self.assertRaises(X509.X509Error):
-            X509.load_request_string('Hello')
-        with self.assertRaises(X509.X509Error):
-            X509.load_request_der_string('Hello')
-        with self.assertRaises(X509.X509Error):
-            X509.load_crl('tests/alltests.py')
+        try:
+            with self.assertRaises(X509.X509Error):
+                X509.load_cert_string('Hello')
+            with self.assertRaises(X509.X509Error):
+                X509.load_cert_der_string('Hello')
+            with self.assertRaises(X509.X509Error):
+                X509.new_stack_from_der(b'Hello')
+            with self.assertRaises(X509.X509Error):
+                X509.load_cert('tests/alltests.py')
+            with self.assertRaises(X509.X509Error):
+                X509.load_request('tests/alltests.py')
+            with self.assertRaises(X509.X509Error):
+                X509.load_request_string('Hello')
+            with self.assertRaises(X509.X509Error):
+                X509.load_request_der_string('Hello')
+            with self.assertRaises(X509.X509Error):
+                X509.load_crl('tests/alltests.py')
+        except SystemError:
+            pass
 
     def test_long_serial(self):
         cert = X509.load_cert('tests/long_serial_cert.pem')
@@ -566,6 +566,12 @@ class X509TestCase(unittest.TestCase):
         self.assertEqual(cert.get_serial_number(),
                          127614157056681299805556476275995414779)
 
+    def test_set_long_serial(self):
+        cert = X509.X509()
+        cert.set_serial_number(127614157056681299805556476275995414779)
+        self.assertEqual(cert.get_serial_number(),
+                         127614157056681299805556476275995414779)
+
     def test_date_after_2050_working(self):
         cert = X509.load_cert('tests/bad_date_cert.crt')
         self.assertEqual(str(cert.get_not_after()), 'Feb  9 14:57:46 2116 GMT')
@@ -576,7 +582,7 @@ class X509TestCase(unittest.TestCase):
         https://github.com/fedora-infra/fedmsg/pull/389
         """
         # Does this raise an exception?
-        cert = X509.load_cert('tests/easy_rsa.pem')
+        X509.load_cert('tests/easy_rsa.pem')
 
 
 class X509StackTestCase(unittest.TestCase):
@@ -586,7 +592,10 @@ class X509StackTestCase(unittest.TestCase):
 
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', DeprecationWarning)
-            seq = base64.decodestring(b64)
+            if six.PY3:
+                seq = base64.decodebytes(b64)
+            else:
+                seq = base64.decodestring(b64)
 
         stack = X509.new_stack_from_der(seq)
         cert = stack.pop()
@@ -606,7 +615,10 @@ class X509StackTestCase(unittest.TestCase):
 
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', DeprecationWarning)
-            seq = base64.decodestring(b64)
+            if six.PY3:
+                seq = base64.decodebytes(b64)
+            else:
+                seq = base64.decodestring(b64)
 
         stack = X509.new_stack_from_der(seq)
         num = len(stack)
@@ -685,10 +697,10 @@ class CRLTestCase(unittest.TestCase):
 
 def suite():
     st = unittest.TestSuite()
-    st.addTest(unittest.makeSuite(X509TestCase))
-    st.addTest(unittest.makeSuite(X509StackTestCase))
-    st.addTest(unittest.makeSuite(X509ExtTestCase))
-    st.addTest(unittest.makeSuite(CRLTestCase))
+    st.addTest(unittest.TestLoader().loadTestsFromTestCase(X509TestCase))
+    st.addTest(unittest.TestLoader().loadTestsFromTestCase(X509StackTestCase))
+    st.addTest(unittest.TestLoader().loadTestsFromTestCase(X509ExtTestCase))
+    st.addTest(unittest.TestLoader().loadTestsFromTestCase(CRLTestCase))
     return st
 
 
